@@ -161,6 +161,12 @@ def exec_dmscript(*scripts: Script,
     mechanism is not. This means you can use any typing as the key but you can 
     only get the value (on python side) with this exact typing.
 
+    Note that there are some variables that will be accessable in the dm-script
+    even if no `setvars` are given. Those are:
+    - `__file__`, string: the location of the dm-script file if the `scripts`
+      is a file path, otherwise it will be "<inline>", this is updated for 
+      every included file
+
     Example
     -------
     >>> prefix = "\n".join((
@@ -230,6 +236,7 @@ def convert_to_taggroup(tags: typing.Union[typing.Dict[str, Convertable],
 
     See
     ---
+    convert_from_taggroup()
     DigitalMicrograph.Py_TagGroup.CopyTagsFrom()
 
     Raises
@@ -300,6 +307,50 @@ def convert_to_taggroup(tags: typing.Union[typing.Dict[str, Convertable],
         
     return tag_group
 
+def convert_from_taggroup(taggroup: DM.Py_TagGroup) -> typing.Union[list, dict]:
+    """Convert the given DigitalMicrograph `taggroup` to a dict or list.
+
+    Note that this function cannot be used to convert global tags. This can 
+    only be used for `Py_TagGroup` objects created in the python code. 
+    Otherwise the reference to the `TagGroup` object is somehow lost.
+
+    See
+    ---
+    convert_to_taggroup()
+
+    Parameters
+    ----------
+    taggroup : DigitalMicrograph.Py_TagGroup
+        The `TagGroup` object created in python
+    
+    Returns
+    -------
+    dict, list
+        A dict or list representing the tag group, the type depends on the 
+        `TagGroup.IsList()` return value
+    """
+
+    tg_type = list if taggroup.IsList() else dict
+    tg_name = "convert_tg_{}".format(int(time.time() * 100))
+
+    # save tag group to persistent tags to pass them to dm-script
+    DM.GetPersistentTagGroup().SetTagAsTagGroup(tg_name, taggroup)
+
+    # copy the tag group to a local variable to sync that variable back to 
+    # python
+    dm_code = "\n".join((
+        "TagGroup {};".format(tg_name),
+        "GetPersistentTagGroup().TagGroupGetTagAsTagGroup(\"{}\", {});".format(tg_name, tg_name)
+    ))
+    
+    with exec_dmscript(dm_code, readvars={tg_name: tg_type}) as script:
+        return script[tg_name]
+    
+    raise RuntimeError(
+        ("Could not find the TagGroup {} in synchronized the dm-script code " + 
+         "that was used to convert a TagGroup to a dict or list.").format(tg_name)
+    )
+
 def get_dm_type(datatype: typing.Union[str, type], 
                 for_taggroup: typing.Optional[bool]=False):
     """Get the dm-script equivalent for the given `datatype`.
@@ -437,11 +488,13 @@ def escape_dm_string(str_content: str):
             .replace("\t", "\\t")
             .replace("\0", "\\0"))
 
-
 class DMScriptWrapper:
     """Wraps one or more dm-scripts.
     """
 
+    debug_start_marker = "@execdmscript.ignore.start"
+    debug_end_marker = "@execdmscript.ignore.end"
+
     def __init__(self,
                  *scripts: Script, 
                  readvars: typing.Optional[dict]=None,
@@ -644,26 +697,39 @@ class DMScriptWrapper:
         )
         
         # add the real code to execute
+        dm_file_added = False
         for i, (kind, script) in enumerate(self.scripts):
             if isinstance(kind, str):
                 kind = kind.lower()
 
             if kind == "file":
                 source = script
+                dm__file__ = script
                 comment = "// File {}".format(escape_dm_string(source))
                 with open(script, "r") as f:
                     code = f.read()
             elif kind == "script":
                 source = "<inline script in parameter {}>".format(i)
                 comment = "// Directly given script"
+                dm__file__ = "<inline>"
                 code = script
 
             dmscript, startpos = self._addCode(
-                dmscript, comment, "<comments>", None, startpos
+                dmscript, 
+                DMScriptWrapper.getDMCodeForVariable("__file__", dm__file__, not dm_file_added), 
+                "<__file__>", 
+                None, 
+                startpos
             )
             dmscript, startpos = self._addCode(
-                dmscript, code, source, script, startpos
+                dmscript, comment, "<comments>", None, startpos, True
             )
+            dmscript, startpos = self._addCode(
+                dmscript, code, source, script, startpos, True
+            )
+
+            if not dm_file_added:
+                dm_file_added = True
 
         wait_for_signals = []
         # execute in a separate thread
@@ -689,6 +755,8 @@ class DMScriptWrapper:
                     source = "<separate_thread parameter {}>".format(i)
                     comment = "// Directly given script"
                     code = script
+                
+                code = code.encode("iso-8859-1")
 
                 dmscript, startpos = self._addCode(
                     dmscript, comment, "<comments>", None, startpos
@@ -720,10 +788,16 @@ class DMScriptWrapper:
     
     def _addCode(self, dmscript: list, code: typing.Union[list, tuple, str], 
                  origin_name: str, origin_detail: typing.Any,
-                 startpos: typing.Optional[int]=1) -> typing.Tuple[list, int]:
+                 startpos: typing.Optional[int]=1,
+                 remove_debug_lines: typing.Optional[bool]=True) -> typing.Tuple[list, int]:
         """Add the `code` to the `dmscript` and save its position in 
         `DMScriptWrapper._script_sources`.
 
+        Raises
+        ------
+        ValueError
+            When the `code` neither is a string nor a list nor a tuple
+
         Parameters
         ----------
         dmscript : list
@@ -739,6 +813,9 @@ class DMScriptWrapper:
             only
         startpos : int, optional
             The line where the code starts, default: 1
+        remove_debug_lines : bool, optional
+            Whether to remove all lines between the debug start marker and the
+            debug end marker
         
         Returns
         -------
@@ -746,19 +823,36 @@ class DMScriptWrapper:
             The dmscript list with the added code fragment at index 0, the 
             start position of the next code fragment at index 1
         """
+
         # the number of lines (minus one)
         code_lines = 0
         if isinstance(code, (list, tuple)):
             code_lines = sum([c.count("\n") for c in code]) + len(code) - 1
-            dmscript += code
+            code = "\n".join(code)
         elif isinstance(code, str):
             code_lines = code.count("\n")
-            dmscript.append(code)
         else:
             raise ValueError(("The code neither is a list, nor a tuple nor " + 
                               "nor a string but a '{}' which is not " + 
                               "supported").format(type(code)))
 
+        if remove_debug_lines:
+            comment_lines = False
+            lines = code.split("\n")
+            for i, line in enumerate(lines):
+                if DMScriptWrapper.debug_start_marker in line:
+                    comment_lines = True
+                
+                if comment_lines:
+                    lines[i] = "// " + line
+                
+                if DMScriptWrapper.debug_end_marker in line:
+                    comment_lines = False
+            
+            code = "\n".join(lines)
+
+        dmscript.append(code)
+
         self._script_sources.append({
             "start": startpos, 
             "end": startpos + code_lines,
@@ -1269,7 +1363,8 @@ class DMScriptWrapper:
             )
     
     @staticmethod
-    def getDMCodeForVariable(name: str, value: Convertable):
+    def getDMCodeForVariable(name: str, value: Convertable, 
+                             declare_type: typing.Optional[bool]=True):
         """Create the dm-script code for defining and declaring a variable.
 
         Parameters
@@ -1279,6 +1374,8 @@ class DMScriptWrapper:
             start with a digit
         value : int, float, str, boolean, dict, list, tuple or None
             The value to set
+        declare_type : bool, optional
+            Whether to declare the type or not, default: True
         
         Returns
         -------
@@ -1291,20 +1388,28 @@ class DMScriptWrapper:
         dm_type = get_dm_type(py_type, for_taggroup=False)
 
         if py_type in (dict, list, tuple):
-            dmscript += DMScriptWrapper._getTagGroupDMCodeForVariable(name, value)
+            dmscript += DMScriptWrapper._getTagGroupDMCodeForVariable(
+                name, value, declare_type
+            )
         else:
             if isinstance(value, str):
                 value = "\"{}\"".format(escape_dm_string(value))
             
-            dmscript.append("{} {} = {};".format(dm_type, 
-                                                 escape_dm_variable(name), 
-                                                 value))
+            if declare_type:
+                line = "{type} {var} = {val};"
+            else:
+                line = "{var} = {val};"
+            
+            dmscript.append(line.format(type=dm_type, 
+                                        var=escape_dm_variable(name), 
+                                        val=value))
         
         return "\n".join(dmscript)
     
     @staticmethod
     def _getTagGroupDMCodeForVariable(name: str, 
-                                      value: Convertable, 
+                                      value: Convertable,
+                                      declare_type: typing.Optional[bool]=True,
                                       prefix: typing.Optional[str]="",
                                       depth: typing.Optional[int]=0):
         """Create the dm-script code for defining and declaring a `TagGroup` or
@@ -1317,6 +1422,12 @@ class DMScriptWrapper:
             start with a digit
         value : dict, list, tuple
             The dict, list or tuple to express as a `TagGroup` or `TagList`
+        declare_type : bool, optional
+            Whether to declare the type or not, default: True
+        prefix : str, optional
+            A prefix to add before the `TagGroup` variable name
+        depth : int, optional
+            The current depth
         
         Returns
         -------
@@ -1336,9 +1447,14 @@ class DMScriptWrapper:
             iterator = enumerate(value)
         
         # set variable
-        dmscript.append("{} {} = {};".format(dm_type, 
-                                             escape_dm_variable(prefix + name), 
-                                             creator))
+        if declare_type:
+            line = "{type} {var} = {val};"
+        else:
+            line = "{var} = {val};"
+        
+        dmscript.append(line.format(type=dm_type, 
+                                    var=escape_dm_variable(prefix + name), 
+                                    val=creator))
 
         index = None
         if py_type == dict:
@@ -1372,7 +1488,7 @@ class DMScriptWrapper:
                 p = "__exec_dmscript_"
                 n = "{}_tg_{}_{}".format(escape_dm_variable(name), i, depth + 1)
                 dmscript += DMScriptWrapper._getTagGroupDMCodeForVariable(
-                    n, value, p, depth + 1
+                    n, value, True, p, depth + 1
                 )
                 # rewrite the value to the variable name of the TagGroup or 
                 # TagList that now exists
